from construct import *

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

__all__ = ["Command", "Reset", "Frequency", "WRate", "Bandwidth", "Deviation", "Power", "Rate", "Config"]

class Command(object):
    def __init__(self, request, response, **kwargs):
        self.request = request
        self.response = response
        self.kwargs = kwargs

    def encode(self, stream):
        return self.request.build_stream(Container(**(self.kwargs)), stream)

    def decode(self, stream):
        return self.response.parse_stream(stream)

class PowerAdapter(Adapter):
    power = [1,2,5,8,11,14,17,20]
    def _decode(self, obj, ctx):
        return PowerAdapter.power[obj]
    def _encode(self, obj, ctv):
        tmp = [i for (i,x) in enumerate(PowerAdapter.power) if x >= obj]
        if len(tmp):
            return tmp[0]
        return len(PowerAdapter.power)-1

frequency = UBInt32("frequency")
wrate     = UBInt32("wrate")
bandwidth = UBInt16("bandwidth")
deviation = UBInt8("deviation")
power     = PowerAdapter(UBInt8("power"))
rate      = UBInt32("rate")

no_response = Struct("ok", Magic('OK\r\n'))
header = Magic('\xAA\xFA')

config = Struct("config", frequency, wrate, bandwidth, deviation, power, rate)

def make_command(magic, *fields):
    class Req(Command):
        def __init__(self, **kwargs):
            req = Struct("request", header, Magic(magic), *fields)
            super(Req, self).__init__(req, no_response, **kwargs)
    return Req

Reset     = make_command('\xF0')
Frequency = make_command('\xD2', frequency)
WRate     = make_command('\xC3', wrate)
Bandwidth = make_command('\xB4', bandwidth)
Deviation = make_command('\xA5', deviation)
Power     = make_command('\x96', power)
Rate      = make_command('\x1E', rate)

class Config(Command):
    def __init__(self):
        req = Struct("config", header, Magic('\xE1'))
        super(Config, self).__init__(req, config)




import serial

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

__all__ = ["Client"]

class Client(object):
    def __init__(self, **kwargs):
        self.hdl = serial.Serial(**kwargs)

    def __call__(self, command):
        command.encode(self.hdl)
        return command.decode(self.hdl)




#!/usr/bin/python

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from optparse import OptionParser
from hytrp.client import *
from hytrp.protocol import *

parser = OptionParser()
parser.add_option("-d", "--device", dest="device", metavar="FILE", help="tty device to communicate", default="/dev/ttyS0")
parser.add_option("-r", "--baudrate", dest="rate", metavar="RATE", help="baudrate for device", default=9600, type="int")
parser.add_option("-q", "--quite", dest="quite", default=False, help="don't print output", action="store_true")
parser.add_option("--reset", dest="reset", default=False, help="reset to factory defaults", action="store_true")
parser.add_option("--frequency", dest="frequency", metavar="FREQ", help="set working frequency, Hz", type="int")
parser.add_option("--power", dest="power", metavar="PWR", help="set working power, dBm", type="int")
parser.add_option("--newrate", dest="newrate", metavar="RATE", help="set new serial rate, bps", type="int")
parser.add_option("--wrate", dest="wrate", metavar="RATE", help="set wireless rate, bps", type="int")
parser.add_option("--bandwidth", dest="bandwidth", metavar="FREQ", help="set bandwidth, KHz", type="int")
parser.add_option("--deviation", dest="deviation", metavar="FREQ", help="set frequency deviation, KHz", type="int")

(options, args) = parser.parse_args()

c = Client(port = options.device, baudrate = options.rate, timeout = 5)
if options.reset:
    c(Reset())
if options.frequency:
    c(Frequency(frequency=options.frequency))
if options.power:
    c(Power(power=options.power))
if options.newrate:
    c(Rate(rate=options.newrate))
if options.wrate:
    c(WRate(wrate=options.wrate))
if options.bandwidth:
    c(Bandwidth(bandwidth=options.bandwidth))
if options.deviation:
    c(Deviation(deviation=options.deviation))

if not options.quite:
    fmt = """
Working frequency:   %d Hz
Wireless data rate:  %d bps
Receiving bandwidth: %d KHz
Frequency deviation: %d KHz
Transmission power:  %d dBm
Serial data rate:    %d bps
    """
    current = c(Config())
    print fmt % (current['frequency'], current['wrate'], current['bandwidth'], current['deviation'], current['power'], current['rate'])



#!/usr/bin/env python

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4

from distutils.core import setup

setup(name='pyhytrp',
    version='0.1',
    description='Python interface for configuring HYTRP serial devices by Canton Electronics',
    author='Matwey V. Kornilov',
    author_email='matwey.kornilov@gmail.com',
    url='https://bitbucket.org/matwey/pyhytrp',
    packages=['hytrp'],
    scripts=['hytrpctl.py'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Topic :: System :: Hardware',]
    )
